
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content.Pipeline.Graphics;
#endregion

namespace WindowsGame1
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;
        Model model;
        Matrix world = Matrix.CreateRotationX(MathHelper.ToRadians(-10.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(-15.0f));
        Matrix view = Matrix.CreateLookAt(new Vector3(0.0f, 0.0f, 10.0f), Vector3.Zero, Vector3.Up);
        Effect effect;

        readonly float rotationSpeed = MathHelper.ToRadians(90.0f); // Radiant pro Sekunde
        readonly float translationSpeed = 10.0f; // Lngeneinheiten pro Sekunde

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);

            Window.AllowUserResizing = true;
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // TODO: Load any ResourceManagementMode.Automatic content

                model = content.Load<Model>("model");

                effect = content.Load<Effect>("effect");
                effect.CurrentTechnique = effect.Techniques["phong"];

                effect.Parameters["ColorTexture"].SetValue(content.Load<Texture2D>("checker"));

                foreach (ModelMesh mm in model.Meshes)
                {
                    foreach (ModelMeshPart mmp in mm.MeshParts)
                    {
                        mmp.Effect = effect;
                    }
                }
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here

            if (Keyboard.GetState().IsKeyDown(Keys.Left))
            {
                world = world * Matrix.CreateRotationY((float)(-rotationSpeed * gameTime.ElapsedGameTime.TotalSeconds));
            }
            if (Keyboard.GetState().IsKeyDown(Keys.Right))
            {
                world = world * Matrix.CreateRotationY((float)(rotationSpeed * gameTime.ElapsedGameTime.TotalSeconds));
            }
            if (Keyboard.GetState().IsKeyDown(Keys.Up))
            {
                world = world * Matrix.CreateRotationX((float)(-rotationSpeed * gameTime.ElapsedGameTime.TotalSeconds));
            }
            if (Keyboard.GetState().IsKeyDown(Keys.Down))
            {
                world = world * Matrix.CreateRotationX((float)(rotationSpeed * gameTime.ElapsedGameTime.TotalSeconds));
            }
            if (Keyboard.GetState().IsKeyDown(Keys.PageUp))
            {
                view = Matrix.CreateTranslation(0.0f, 0.0f, (float)(-translationSpeed * gameTime.ElapsedGameTime.TotalSeconds)) * view;
            }
            if (Keyboard.GetState().IsKeyDown(Keys.PageDown))
            {
                view = Matrix.CreateTranslation(0.0f, 0.0f, (float)(translationSpeed * gameTime.ElapsedGameTime.TotalSeconds)) * view;
            }

            base.Update(gameTime);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {

            // also show the backfaces of the triangles
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;

            // uncomment this to show the triangles
            //graphics.GraphicsDevice.RenderState.FillMode = FillMode.WireFrame;

            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here

            Matrix projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(50.0f),
                graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height,
                0.1f, 100.0f);

            effect.Parameters["WorldViewProj"].SetValue(world * view * projection);
            effect.Parameters["World"].SetValue(world);
            effect.Parameters["WorldIT"].SetValue(Matrix.Transpose(Matrix.Invert(world)));
            effect.Parameters["ViewI"].SetValue(Matrix.Invert(view));
            effect.Parameters["DirToLight"].SetValue(Vector3.Normalize(new Vector3(-1.0f, 1.0f, 1.0f)));
            
            foreach (ModelMesh mm in model.Meshes)
            {
                mm.Draw();
            }

            base.Draw(gameTime);
        }
    }
}