uniform extern float4x4 WorldViewProj : WORLDVIEWPROJECTION;
uniform extern float4x4 World : WORLD;
uniform extern float4x4 WorldIT : WORLDINVERSETRANSPOSE;
uniform extern float4x4 ViewI : VIEWINVERSE;
uniform extern float3 DirToLight;
uniform extern texture ColorTexture;
uniform extern texture NormalTexture;

sampler colorSampler = sampler_state 
{
    texture = <ColorTexture>;
    AddressU  = WRAP;        
    AddressV  = WRAP;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

struct VS_OUTPUT
{
    float4 position : POSITION;
    float3 normal : TEXCOORD0;
    float3 view : TEXCOORD1;
    float2 uv : TEXCOORD2;
};

VS_OUTPUT Transform(
    float4 Pos  : POSITION, 
    float3 Norm : NORMAL,
    float2 uv : TEXCOORD0 )
{
    VS_OUTPUT Out;
    Out.position = mul(Pos, WorldViewProj);
    Out.normal = normalize(mul(Norm, WorldIT));
    float3 worldEyePos = ViewI[3].xyz;
    float3 worldVertPos = mul(Pos, World).xyz;
    Out.view = worldEyePos - worldVertPos; // nicht normalisieren, sonst falsche Interpolation
    Out.uv = uv;
    return Out;
}

float4 PixelShader( VS_OUTPUT vsout ) : COLOR
{
	float3 normal = normalize(vsout.normal);
	float3 view = normalize(vsout.view);
	
	// Die HLSL-Funktion "lit" kann alles dies in einer Zeile.
	float3 halfvec = normalize(view + DirToLight);
	float specular = dot(normal, halfvec);
	if(specular < 0)
	{
		specular = 0;
	}
	float diffuse = dot(normal, DirToLight);
	if(diffuse < 0)
	{
		diffuse = 0;
		specular = 0;
	}
    return diffuse*tex2D(colorSampler, vsout.uv) + pow(specular, 100.0)*float4(1,1,1,1);
}

struct VSHALO_OUTPUT
{
    float4 position : POSITION;
};

VSHALO_OUTPUT TransformHalo(
    float4 Pos  : POSITION, 
    float3 Norm : NORMAL,
    float2 uv : TEXCOORD0 )
{
    VSHALO_OUTPUT Out;
    Out.position = mul(Pos*float4(1.2, 1.2, 1.2, 1.0), WorldViewProj);
    return Out;
}

float4 PixelShaderHalo( VS_OUTPUT vsout ) : COLOR
{
    return float4(1, 0, 0, 0.5);
}

technique phong
{
    pass P0
    {
	    AlphaBlendEnable = false;
		ZWriteEnable = true;   	    
        vertexShader = compile vs_2_0 Transform();
        pixelShader = compile ps_2_0 PixelShader();    
    }
    
    pass P1
    {
		AlphaBlendEnable = true;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;
		ZWriteEnable = false;		
        vertexShader = compile vs_2_0 TransformHalo();
        pixelShader = compile ps_2_0 PixelShaderHalo();             
    }
}